# Time
import time


def sleep(seconds: int = 0):
    """The Python sleep () time method suspends execution for the 
    given number of seconds. The argument can be a floating point 
    number to indicate a more accurate sleep time.

    :param seconds: Total of seconds that the program will be sleeping
    """
    time.sleep(seconds)
